/**************************************************************************
   STV-TOOLS: Shannon Total Variation based image processing
   Copyright (C) 2016 Remy Abergel (remy.abergel AT gmail.com),
   Lionel Moisan (Lionel.Moisan AT parisdescartes.fr).

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

/*

  Compilation (with gcc): gcc -O3 rectangularmask.c tiffwrite.c -lm -ltiff -o rectangularmask

  References :

  [1] R. Abergel and L. Moisan, ``The Shannon Total Variation'', preprint MAP5 n° 2016-19, July 2016.
  [2] L. Moisan. How to discretize the total variation of an image? Proceedings in Applied Mathematics and Mechanics, 2007.

  Versions :

  + v 1.0 : (09/2016) initial version (Rémy Abergel)

 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "fftw3.h"

#define FREE_ALL() {					\
    if(NULL != u) free(u);				\
  }
#define ASSERT_ALLOC(cmd) if(NULL == (cmd)) { printf("Not enough memory\n"); FREE_ALL(); exit(EXIT_FAILURE); }

static void display_usage()
{
  printf("\nCompute a rectangular mask (rectangle center = image center).\n\n");
  printf("Usage: rectangularmask Nx Ny nx ny out\n\n");
  printf("   Nx  : width of the output mask\n");
  printf("   Ny  : height of the output mask\n");
  printf("   nx  : rectangle width\n");
  printf("   ny  : rectangle height\n");
  printf("   out : output mask (TIFF)\n\n");
}


int main(int argc, char **argv)
{
  char *filename_out = NULL, *Nxvalue = NULL, *Nyvalue = NULL, *nxvalue = NULL, *nyvalue = NULL,vflag,fflag;
  double *u = NULL;
  int i,Nx,Ny,nx,ny,x,y,adr,err;
  vflag = fflag = 0;

  /******************************************/
  /*            arguments parser            */
  /******************************************/

  /* main parser */
  for(i=1;i<argc;i++) {
    if (NULL == Nxvalue) {
      Nxvalue = argv[i]; err = sscanf(Nxvalue,"%d",&Nx);
      if(err != 1) { display_usage(); printf("Error: could not retrieve properly input 'Nx'\n\n"); return EXIT_FAILURE; }
    }
    else if (NULL == Nyvalue) {
      Nyvalue = argv[i]; err = sscanf(Nyvalue,"%d",&Ny);
      if(err != 1) { display_usage(); printf("Error: could not retrieve properly input 'Ny'\n\n"); return EXIT_FAILURE; }
    }
    else if (NULL == nxvalue) {
      nxvalue = argv[i]; err = sscanf(nxvalue,"%d",&nx);
      if(err != 1) { display_usage(); printf("Error: could not retrieve properly input 'nx'\n\n"); return EXIT_FAILURE; }
    }
    else if (NULL == nyvalue) {
      nyvalue = argv[i]; err = sscanf(nyvalue,"%d",&ny);
      if(err != 1) { display_usage(); printf("Error: could not retrieve properly input 'ny'\n\n"); return EXIT_FAILURE; }
    }
    else if (NULL == filename_out) filename_out = argv[i];
    else {
      display_usage();
      printf("Error: could not retrieve properly input arguments.\n\n");
      return EXIT_FAILURE;
    }
  }

  /* check consistency */
  if ((NULL == filename_out) || (NULL == Nxvalue) || (NULL == Nyvalue) || (NULL == nxvalue) || (NULL == nyvalue)) {
    display_usage();
    if (NULL == filename_out) printf("Error: output 'out' is missing\n\n");
    else if (NULL == Nxvalue) printf("Error: input 'Nx' is missing\n\n");
    else if (NULL == Nyvalue) printf("Error: input 'Ny' is missing\n\n");
    else if (NULL == nxvalue) printf("Error: input 'nx' is missing\n\n");
    else printf("Error: input 'ny' is missing\n\n");
    return EXIT_FAILURE;
  }

  /*********************/
  /* Memory allocation */
  /*********************/
  ASSERT_ALLOC(u = (double*) calloc (Nx*Ny , sizeof(double)));



  /***********************************/
  /*      KERNEL: compute mask       */
  /***********************************/
  for(x=-nx/2;x<=(nx-1)/2;x++)
    for(y=-ny/2;y<=ny/2;y++) {
      u[(y+Ny/2)*Nx+(x+Nx/2)] = 1;
    }
  /***********************************/
  /*          END OF KERNEL          */
  /***********************************/



  /********************************/
  /* Write output as a TIFF Image */
  /********************************/
  if (EXIT_FAILURE == save_tiff(filename_out,u,Nx,Ny,8,fflag,1,vflag)) {
    printf("An error occured during the TIFF image saving process.\n");
    printf("Free memory and exit the program.\n\n");
    FREE_ALL();
    return EXIT_FAILURE;
  }

  /* free memory */
  FREE_ALL();
}
